#!/usr/bin/env python3
# -*- coding:utf-8 -*-
#
# @Time    : 2025/07/28 18:21
# @Author  : Wu_RH
# @FileName: 3N.py
"""
[4N]邻居：将每个雷向最近的雷连有向边（如果离一个雷最近的雷有多个，就都连），与一个雷连接的边竖无限制，雷线索表示从该雷开始走，只能沿着有向边走，走到已经走过的雷经过的边数的最小值。
"""
